"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
const aurelia_event_aggregator_1 = require("aurelia-event-aggregator");
const aurelia_framework_1 = require("aurelia-framework");
const utils_1 = require("../utility/utils");
const app_configuration_1 = require("./app-configuration");
const view_1 = require("./view");
const _ROUTER_CONFIGURED_CALLBACK_NAME = "routerConfigured";
const _ONLINE_CALLBACK_NAME = "onOnline";
const _OFFLINE_CALLBACK_NAME = "onOffline";
const _APP_VIEW_HTML_ELEMENT_NAME = "pitaya-app-view";
const _TRANSITION_TOP_LEVEL = "pitaya-page-transition--top-level";
const _TRANSITION_TOP_TO_CHILD = "pitaya-page-transition--top-to-child";
const _TRANSITION_CHILD_TO_TOP = "pitaya-page-transition--child-to-top";
const _META_THEME_COLOR_SELECTOR = "meta[name=theme-color]";
const _META_THEME_COLOR_ELEMENT_STRING = "<meta name='theme-color'>";
const _privateProperties = {
    currentInstance: null,
    backButtonOverrideHandler: null,
    router: null,
    routerConfig: null,
    themeColorMetaElement: document.querySelector(_META_THEME_COLOR_SELECTOR) || document.head.appendChild(utils_1.createElement(_META_THEME_COLOR_ELEMENT_STRING)),
    templateViewModel: null
};
let PitayaApp = class PitayaApp {
    constructor(_eventAggregator) {
        this._eventAggregator = _eventAggregator;
        this._settings = {
            title: "",
            configuration: {},
            routes: null,
            viewTransition: {
                childToTop: {
                    className: _TRANSITION_CHILD_TO_TOP,
                    swapOrder: "with"
                },
                topToChild: {
                    className: _TRANSITION_TOP_TO_CHILD,
                    swapOrder: "with"
                },
                topToTop: {
                    className: _TRANSITION_TOP_LEVEL,
                    swapOrder: "with"
                }
            }
        };
        _privateProperties.currentInstance = this;
        _wrapMethod.call(this, this._activate, "activate");
        _wrapMethod.call(this, this._created, "created");
        _wrapMethod.call(this, this._bind, "bind");
    }
    static get instance() {
        return _privateProperties.currentInstance;
    }
    get enableControls() {
        return _privateProperties.templateViewModel.interactive;
    }
    set enableControls(value) {
        _privateProperties.templateViewModel.interactive = value;
    }
    get currentView() {
        return view_1.PitayaView.activeInstance || _privateProperties.router.currentInstruction.viewPortInstructions.default.component.viewModel;
    }
    get isOnline() {
        return _privateProperties.templateViewModel.isOnline;
    }
    get statusBarColor() {
        return _privateProperties.themeColorMetaElement.getAttribute("content");
    }
    set statusBarColor(value) {
        _privateProperties.themeColorMetaElement.setAttribute("content", value);
    }
    async configureRouter(config, router) {
        const routs = await this._getRoutes();
        const [mainRoute] = routs;
        config.title = this._settings.title;
        config.map(routs);
        config.mapUnknownRoutes(mainRoute.moduleId);
        _privateProperties.router = router;
        _privateProperties.routerConfig = config;
        this._setRouteChangeHandler(config);
        this[_ROUTER_CONFIGURED_CALLBACK_NAME] && this[_ROUTER_CONFIGURED_CALLBACK_NAME](config, router);
    }
    navigate(routeName, options) {
        return _privateProperties.router.navigate(routeName, options);
    }
    navigateToRoute(routeName, params, options) {
        return _privateProperties.router.navigateToRoute(routeName, params, options);
    }
    navigateBack() {
        _privateProperties.router.navigateBack();
    }
    async _activate() {
        await this._initSettings();
    }
    _created(parentContext, context) {
        const model = context.controllers.find(controller => controller.behavior.elementName === _APP_VIEW_HTML_ELEMENT_NAME);
        if (!model) {
            throw new Error(`[PitayaView] - Can not find <${_APP_VIEW_HTML_ELEMENT_NAME}> custom element.`);
        }
        _privateProperties.templateViewModel = model.viewModel;
    }
    _bind() {
        this._initNetworkChangeHandler();
    }
    _setRouteChangeHandler(config) {
        this._eventAggregator.subscribe("router:navigation:processing", this._handleRouteProcessing.bind(this));
        this._eventAggregator.subscribe("router:navigation:success", this._handleRouteSuccess.bind(this));
    }
    _handleRouteProcessing({ instruction }) {
        if (!instruction.previousInstruction) {
            return;
        }
        const { viewTransition } = this._settings;
        const oldOrder = 1 * (instruction.previousInstruction.config.navModel.order || 0);
        const newOrder = 1 * (instruction.config.navModel.order || 0);
        if (oldOrder > newOrder) {
            _privateProperties.templateViewModel.transitionAnimationClass = viewTransition.childToTop.className;
            _privateProperties.templateViewModel.routerSwapOrder = viewTransition.childToTop.swapOrder;
        }
        else if (oldOrder < newOrder) {
            _privateProperties.templateViewModel.transitionAnimationClass = viewTransition.topToChild.className;
            _privateProperties.templateViewModel.routerSwapOrder = viewTransition.topToChild.swapOrder;
        }
        else {
            _privateProperties.templateViewModel.transitionAnimationClass = viewTransition.topToTop.className;
            _privateProperties.templateViewModel.routerSwapOrder = viewTransition.topToTop.swapOrder;
        }
    }
    _handleRouteSuccess(event) {
        _privateProperties.templateViewModel.currentViewName = _privateProperties.router.currentInstruction.config.name;
    }
    async _initSettings() {
        if (!this._settings.configuration) {
            return;
        }
        let settings;
        if (typeof this._settings.configuration === "string") {
            const file = await fetch(this._settings.configuration);
            settings = await file.json();
        }
        app_configuration_1.PitayaAppConfiguration.set(settings || this._settings.configuration);
    }
    async _getRoutes() {
        if (typeof this._settings.routes === "string") {
            const file = await fetch(this._settings.routes);
            const { routs } = await file.json();
            return routs;
        }
        return this._settings.routes;
    }
    _initNetworkChangeHandler() {
        _privateProperties.templateViewModel.isOnline = navigator.onLine;
        window.addEventListener("online", this._onOnline.bind(this));
        window.addEventListener("offline", this._onOffline.bind(this));
    }
    async _onOnline() {
        _privateProperties.templateViewModel.isOnline = true;
        const { currentView } = this;
        if (_ONLINE_CALLBACK_NAME in currentView) {
            const stopPropagation = currentView[_ONLINE_CALLBACK_NAME]();
            if (stopPropagation) {
                return;
            }
        }
        (_ONLINE_CALLBACK_NAME in this) && this[_ONLINE_CALLBACK_NAME]();
    }
    async _onOffline() {
        _privateProperties.templateViewModel.isOnline = false;
        const { currentView } = this;
        if (_OFFLINE_CALLBACK_NAME in currentView) {
            const stopPropagation = currentView[_OFFLINE_CALLBACK_NAME]();
            if (stopPropagation) {
                return;
            }
        }
        (_OFFLINE_CALLBACK_NAME in this) && this[_OFFLINE_CALLBACK_NAME]();
    }
};
PitayaApp = __decorate([
    aurelia_framework_1.autoinject(),
    __metadata("design:paramtypes", [aurelia_event_aggregator_1.EventAggregator])
], PitayaApp);
exports.PitayaApp = PitayaApp;
PitayaApp.prototype["activate"] = function (...args) { };
PitayaApp.prototype["created"] = function (...args) { };
PitayaApp.prototype["bind"] = function (...args) { };
const _noop = () => { };
const _wrapMethod = function (shadowMethod, methodName) {
    const originalMethod = this[methodName] || _noop;
    this.constructor.prototype[methodName] = (...args) => {
        shadowMethod.call(this, ...args);
        originalMethod.call(this, ...args);
    };
};
exports.overrideBrowserBackButton = function (callback) {
    if (_privateProperties.backButtonOverrideHandler) {
        window.removeEventListener("popstate", _privateProperties.backButtonOverrideHandler);
        _privateProperties.backButtonOverrideHandler = null;
    }
    if (!callback) {
        window.history.back();
        return;
    }
    const handler = function () {
        window.history.pushState({}, "");
        callback();
    };
    _privateProperties.backButtonOverrideHandler = handler;
    window.addEventListener("popstate", handler);
    window.history.pushState({}, "");
};
exports.setStatusBarColor = function (color) {
    _privateProperties.currentInstance.statusBarColor = color;
};
exports.navigate = function (routeName, options) {
    return _privateProperties.currentInstance.navigate.call(_privateProperties.currentInstance, routeName, options);
};
exports.navigateToRoute = function (routeName, params, options) {
    return _privateProperties.currentInstance.navigateToRoute.call(_privateProperties.currentInstance, routeName, params, options);
};
exports.navigateBack = function () {
    _privateProperties.currentInstance.navigateBack.call(_privateProperties.currentInstance);
};
exports.enableControls = function (enable = true) {
    _privateProperties.currentInstance.enableControls = enable;
};
//# sourceMappingURL=app.js.map