"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aurelia_router_1 = require("aurelia-router");
const app_1 = require("./app");
const app_configuration_1 = require("./app-configuration");
const _VIEW_ELEMENT_NAME = "pitaya-view";
const _VIEW_ACTIVE_LEAVE_CLASS = "au-leave-active";
const _ANIMATED_CALLBACK_NAME = "animated";
let _currentViewInstance = null;
class PitayaView {
    constructor() {
        this._settings = {
            hierarchy: 0,
            theme: app_configuration_1.PitayaAppConfiguration.View.defaultTheme,
            redirect: {
                route: null,
                condition: null
            },
        };
        _wrapMethod.call(this, this._canActivate, "canActivate");
        _wrapMethod.call(this, this._activate, "activate");
        _wrapMethod.call(this, this._created, "created");
        _wrapMethod.call(this, this._attached, "attached");
    }
    static get activeInstance() {
        return _currentViewInstance;
    }
    get enableScrolling() {
        return this._templateViewModel.scrollingIsEnabled;
    }
    set enableScrolling(value) {
        this._templateViewModel.scrollingIsEnabled = value;
    }
    navigate(route, options) {
        return app_1.navigate(route, options);
    }
    navigateToRoute(route, params, options) {
        return app_1.navigateToRoute(route, params, options);
    }
    navigateBack() {
        app_1.navigateBack();
    }
    scrollTo(target, offset = 16) {
        const viewElement = this._templateViewModel.rootElement;
        const position = target instanceof HTMLElement ? Math.max(0, target.offsetTop - offset) : target;
        viewElement.scrollTo({
            behavior: "smooth",
            left: 0,
            top: position
        });
    }
    _canActivate(params, routeConfig, navigationInstruction) {
        const { redirect } = this._settings;
        if (!redirect.condition || !redirect.condition.call(this, this, redirect.route)) {
            return true;
        }
        return new aurelia_router_1.RedirectToRoute(redirect.route);
    }
    _activate(params, routeConfig, navigationInstruction) {
        this._settings.hierarchy = routeConfig.nav || 0;
    }
    _created(parentContext, context) {
        const model = context.controllers.find((controller) => {
            const { elementName } = controller.behavior;
            return elementName === _VIEW_ELEMENT_NAME;
        });
        if (!model) {
            throw new Error(`[PitayaView] - Can not find <${_VIEW_ELEMENT_NAME}> custom element.`);
        }
        this._templateViewModel = model.viewModel;
        _currentViewInstance = this;
    }
    _attached() {
        this._setAnimationListener();
    }
    _willBeAnimated() {
        const { rootElement } = this._templateViewModel;
        const siblings = [...rootElement.parentElement.children];
        return siblings.some((sibling) => sibling !== rootElement && sibling.classList.contains(_VIEW_ACTIVE_LEAVE_CLASS));
    }
    _setAnimationListener() {
        const { rootElement } = this._templateViewModel;
        if (!this._willBeAnimated()) {
            this._onAnimationEnd();
            return;
        }
        rootElement.addEventListener("animationend", this._onAnimationEnd.bind(this), { once: true });
    }
    _onAnimationEnd() {
        this._templateViewModel.animationReady = true;
        this[_ANIMATED_CALLBACK_NAME] && this[_ANIMATED_CALLBACK_NAME]();
    }
}
exports.PitayaView = PitayaView;
PitayaView.prototype["canActivate"] = function (...args) { };
PitayaView.prototype["activate"] = function (...args) { };
PitayaView.prototype["created"] = function (...args) { };
PitayaView.prototype["attached"] = function (...args) { };
exports.scrollTo = function (target, offset = 16) {
    _currentViewInstance.scrollTo(target, offset);
};
exports.enableScrolling = function (enable = true) {
    _currentViewInstance.enableScrolling = enable;
};
const _noop = () => { };
const _wrapMethod = function (shadowMethod, methodName) {
    const originalMethod = this[methodName] || _noop;
    this.constructor.prototype[methodName] = (...args) => {
        shadowMethod.call(this, ...args);
        originalMethod.call(this, ...args);
    };
};
//# sourceMappingURL=view.js.map