import { NavigationInstruction, RedirectToRoute, RouteConfig, Router, RouterConfiguration } from "aurelia-router";
import { PitayaView }                                                                       from "./src/app/view";

export interface IPitayaAppViewModel
{
	readonly currentView: PitayaView;

	activate?( params, routeConfig, navigationInstruction ): void | Promise<void>;

	created?( parentContext, context );

	bind?( bindingContext, overrideContext );

	attached?();

	detached?();

	unbind?();

	routerConfigured?( config: RouterConfiguration, router: Router );
}

export interface IPitayaViewModel
{
	readonly enableScrolling: boolean;

	canActivate?( params: any, routeConfig: RouteConfig, navigationInstruction: NavigationInstruction ): boolean | RedirectToRoute | Promise<boolean | RedirectToRoute>;

	activate?( params, routeConfig, navigationInstruction ): void | Promise<void>;

	created?( parentContext, context );

	bind?( bindingContext, overrideContext );

	attached?();

	detached?();

	unbind?();
}

export interface IPitayaComponentViewModel
{
	readonly rootElement: HTMLElement;

	created?( parentContext, context );

	bind?( bindingContext, overrideContext );

	attached?();

	detached?();

	unbind?();

	reinitComponent?();
}

export interface IAppConfig
{
	Development?: {
		enabled?: boolean;
		bodyClassName?: string;
		logging?: "error" | "debug" | "warn" | "info";
	};
	View?: {
		defaultTheme?: string;
	};
}

export interface IAppModelSettings
{
	title: string;
	configuration: string | IAppConfig;
	routes: string | IRoutesConfigRoute[];
	viewTransition: IViewsTransitionModel;
}

export interface IRoutesConfig
{
	routes: IRoutesConfigRoute[];
}

export interface IRoutesConfigRoute
{
	moduleId: string;
	name: string;
	route: string;
	title?: string;
	nav?: number;
}

type OnlyIfDecoratorSignature = ( callback: ( instance, method: string ) => boolean, elseCallbackName?: string ) => void;

export interface IOnlyIfDecorator extends OnlyIfDecoratorSignature
{
	hasParameters: Function;
	hasProperties: Function;
	isOnline: Function;
	underDevelopment: Function;
}

type ViewDecoratorSignature = ( config: IViewDecoratorConfig ) => ( constructor: any ) => any;

export interface IViewDecorator extends ViewDecoratorSignature
{
	redirectTo: RedirectDecoratorFunction;
	theme: SimpleDecoratorFunction;
}

export interface IViewDecoratorConfig
{
	redirectTo?: RedirectDecoratorFunction;
	theme?: string;
}

export interface IPitayaViewSettings
{
	hierarchy: number;
	theme: string;
	redirect: {
		route: string;
		condition: Function;
	};
}

type AppDecoratorSignature = ( config: IAppDecoratorConfig ) => ( constructor: any ) => any;

export interface IAppDecorator extends AppDecoratorSignature
{
	title?: Function;
	configure?: ( options: string | IAppConfig ) => any;
	routes?: ( routes: string | IRoutesConfigRoute[] ) => any;
	viewTransition?: ( model: IViewsTransitionModel ) => any;
}

export interface IAppDecoratorConfig
{
	title?: string;
	configure?: string | IAppConfig;
	routes?: string | IRoutesConfigRoute[];
	viewTransition?: IViewsTransitionModel;
}

type SimpleDecoratorFunction = ( constructor: any ) => any;
type RedirectDecoratorFunction = ( route: string, callback: ( instance, route ) => boolean ) => ( constructor: any ) => any;

export interface IDataFlowConnector
{
	name?: string;

	initialize(): Promise<any>;

	destroy(): Promise<any>;
}

export interface IViewsTransitionModel
{
	topToTop?: IViewsTransition;
	topToChild?: IViewsTransition;
	childToTop?: IViewsTransition;
}

export interface IViewsTransition
{
	swapOrder: string;
	className: string;
}